from izihawa_utils.exceptions import BaseError


class PdfProcessingError(BaseError):
    pass


class TooBigDifferenceAfterCompressionError(PdfProcessingError):
    pass


class EmptyTextError(PdfProcessingError):
    pass


class PyPdfError(PdfProcessingError):
    def __init__(self, nested_error):
        self.nested_error = nested_error



import re


class TextCollector:
    def __init__(self, inverted=False, separator=""):
        self.text = []
        self.operations = []
        self.inverted = inverted
        self.separator = separator

    def add_piece(self, text, operation):
        if self.inverted:
            self.text = [text] + self.text
            self.operations = [operation] * (
                len(text) + len(self.separator)
            ) + self.operations
        else:
            self.text.append(text)
            self.operations.extend([operation] * (len(text) + len(self.separator)))

    def match(self, pattern):
        operations = []
        texts = []

        for match in re.finditer(pattern, self.separator.join(self.text)):
            left, right = match.span()
            texts.append(match.group())
            operations.extend(self.operations[left:right])
        return "\n".join(texts), list(sorted(set(operations)))

    def clear(self):
        self.text = []
        self.operations = []



import tempfile

import ebooklib
from bs4 import BeautifulSoup
from ebooklib import epub


def chapter_to_str(chapter):
    soup = BeautifulSoup(chapter.get_body_content(), "lxml")
    text = [para.get_text() for para in soup.find_all("p")]
    return " ".join(text)


def extract_epub(content: bytes):
    with tempfile.NamedTemporaryFile(mode="w+b", delete=False) as t_file:
        t_file.write(content)
        file_name = t_file.name
        book = epub.read_epub(file_name)
        items = list(book.get_items_of_type(ebooklib.ITEM_DOCUMENT))
        texts = []
        for item in items:
            if "chapter" in item.get_name():
                texts.append(chapter_to_str(item))
        return "".join(texts)
